/*-----------------------------------------------------------------------------
 * Name:    Joystick_MCB1500.c
 * Purpose: Joystick interface for MCB1500 evaluation board
 * Rev.:    1.0.0
 *----------------------------------------------------------------------------*/

/* Copyright (c) 2013 - 2016 ARM LIMITED

   All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
   - Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   - Neither the name of ARM nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.
   *
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
   ---------------------------------------------------------------------------*/

#include "Board_Joystick.h"
#include "Driver_I2C.h"
#include "chip.h"

#define JOYSTICK_COUNT                 (5)

/* Input/output pin description */
typedef struct io_pin {
  uint8_t port;
  uint8_t pin;
} IO_PIN;

/* Joystick pins:
   - center: PIO2_10
   - up:     PIO2_0
   - down:   PIO2_2
   - left:   PIO2_1
   - right:  PIO2_3 */

/* Joystick pin definitions */
const IO_PIN JOYSTICK_PIN[] = {
  { 2, 10 },
  { 2, 0  },
  { 2, 2  },
  { 2, 1  },
  { 2, 3  }
};


/**
  \fn          int32_t Joystick_Initialize (void)
  \brief       Initialize joystick
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t Joystick_Initialize (void) {
  uint32_t n;

  /* Enable IO clocks */
  Chip_Clock_EnablePeriphClock (SYSCTL_CLOCK_IOCON);
  Chip_Clock_EnablePeriphClock (SYSCTL_CLOCK_GPIO2);

  /* Configure pins */
  for (n = 0; n < JOYSTICK_COUNT; n++) {
    Chip_IOCON_PinMuxSet (LPC_IOCON, JOYSTICK_PIN[n].port, JOYSTICK_PIN[n].pin, 0);
    Chip_GPIO_WriteDirBit (LPC_GPIO, JOYSTICK_PIN[n].port, JOYSTICK_PIN[n].pin, 0);
  }
  return 0;
}

/**
  \fn          int32_t Joystick_Uninitialize (void)
  \brief       De-initialize joystick
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t Joystick_Uninitialize (void) {
  uint32_t n;

  /* Unconfigure pins */
  for (n = 0; n < JOYSTICK_COUNT; n++) {
    Chip_IOCON_PinMuxSet (LPC_IOCON, JOYSTICK_PIN[n].port, JOYSTICK_PIN[n].pin, 0);
  }
  return 0;
}

/**
  \fn          uint32_t Joystick_GetState (void)
  \brief       Get joystick state
  \returns     Joystick state
*/
uint32_t Joystick_GetState (void) {
 uint32_t val;

  val  = 0;
  if (!(Chip_GPIO_GetPinState (LPC_GPIO, JOYSTICK_PIN[0].port, JOYSTICK_PIN[0].pin))) val |= JOYSTICK_CENTER;
  if (!(Chip_GPIO_GetPinState (LPC_GPIO, JOYSTICK_PIN[1].port, JOYSTICK_PIN[1].pin))) val |= JOYSTICK_UP;
  if (!(Chip_GPIO_GetPinState (LPC_GPIO, JOYSTICK_PIN[2].port, JOYSTICK_PIN[2].pin))) val |= JOYSTICK_DOWN;
  if (!(Chip_GPIO_GetPinState (LPC_GPIO, JOYSTICK_PIN[3].port, JOYSTICK_PIN[3].pin))) val |= JOYSTICK_LEFT;
  if (!(Chip_GPIO_GetPinState (LPC_GPIO, JOYSTICK_PIN[4].port, JOYSTICK_PIN[4].pin))) val |= JOYSTICK_RIGHT;

  return val;
}
