/*-----------------------------------------------------------------------------
 * Name:    LED_MCB1500.c
 * Purpose: LED interface for MCB1500 evaluation board
 * Rev.:    1.0.0
 *----------------------------------------------------------------------------*/

/* Copyright (c) 2013 - 2016 ARM LIMITED

   All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
   - Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   - Neither the name of ARM nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.
   *
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
   ---------------------------------------------------------------------------*/

#include "Board_LED.h"
#include "chip.h"

#define LED_COUNT                      (4)

/* Input/output pin description */
typedef struct io_pin {
  uint8_t port;
  uint8_t pin;
} IO_PIN;

/* IOCON settings for GPIO pin */
#define PIN_IOCON_GPIO       (IOCON_FUNC0      | \
                              IOCON_DIGMODE_EN | \
                              IOCON_S_MODE_0CLK)

#define PIN_IOCON_GPIO_RST   (IOCON_FUNC0       | \
                              IOCON_MODE_PULLUP | \
                              IOCON_DIGMODE_EN  | \
                              IOCON_S_MODE_0CLK)

/* LED GPIO definitions */
const IO_PIN led[] = {
  { 1, 2 },
  { 1, 3 },
  { 0, 0 },
  { 1, 0 },
};


/**
  \fn          int32_t LED_Initialize (void)
  \brief       Initialize LEDs
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t LED_Initialize (void) {
  uint32_t n;

  /* Enable IO clocks */
  Chip_Clock_EnablePeriphClock (SYSCTL_CLOCK_IOCON);
  Chip_Clock_EnablePeriphClock (SYSCTL_CLOCK_GPIO0);
  Chip_Clock_EnablePeriphClock (SYSCTL_CLOCK_GPIO1);

  /* Configure LED pins as outputs set to high state */
  for (n = 0; n < LED_COUNT; n++) {
    Chip_IOCON_PinMuxSet (LPC_IOCON, led[n].port, led[n].pin, PIN_IOCON_GPIO);
    Chip_GPIO_SetPinState (LPC_GPIO, led[n].port, led[n].pin, true);
    Chip_GPIO_SetPinDIROutput (LPC_GPIO, led[n].port, led[n].pin);
  }

  return 0;
}

/**
  \fn          int32_t LED_Uninitialize (void)
  \brief       De-initialize LEDs
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t LED_Uninitialize (void) {
  uint32_t n;

  /* Reset LED pins to default state */
  for (n = 0; n < LED_COUNT; n++) {
    Chip_GPIO_SetPinDIRInput (LPC_GPIO, led[n].port, led[n].pin);
    Chip_IOCON_PinMuxSet (LPC_IOCON, led[n].port, led[n].pin, PIN_IOCON_GPIO_RST);
  }

  return 0;
}

/**
  \fn          int32_t LED_On (uint32_t num)
  \brief       Turn on requested LED
  \param[in]   num  LED number
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t LED_On (uint32_t num) {

  if (num < LED_COUNT) {
    Chip_GPIO_SetPinState (LPC_GPIO, led[num].port, led[num].pin, false);
  } else {
    return -1;
  }
  return 0;
}

/**
  \fn          int32_t LED_Off (uint32_t num)
  \brief       Turn off requested LED
  \param[in]   num  LED number
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t LED_Off (uint32_t num) {

  if (num < LED_COUNT) {
    Chip_GPIO_SetPinState (LPC_GPIO, led[num].port, led[num].pin, true);
  } else {
    return -1;
  }
  return 0;
}

/**
  \fn          int32_t LED_SetOut (uint32_t val)
  \brief       Write value to LEDs
  \param[in]   val  value to be displayed on LEDs
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t LED_SetOut (uint32_t val) {
  uint32_t n;

  for (n = 0; n < LED_COUNT; n++) {
    if (val & (1 << n)) { LED_On (n); }
    else                { LED_Off(n); }
  }
  return 0;
}

/**
  \fn          uint32_t LED_GetCount (void)
  \brief       Get number of LEDs
  \return      Number of available LEDs
*/
uint32_t LED_GetCount (void) {
  return LED_COUNT;
}
